<?php
// version 3.0
date_default_timezone_set('Europe/Warsaw');

ini_set('display_errors', 1);

// argumenty przekazane do skryptu
$args = array();
if (isset($_SERVER['argv'])) {
    $args = $_SERVER['argv'];
    array_shift($args);
}

final class FileIndexLog {



    private $log = FALSE;



    public function __construct() {

        $this->log = new FileLog(Log::FILE_LOG_WRITER);

    }


    public function write($message, $priority = Log::EVENT_INFO) {

        $this->log->setWriterConfig(Log::FILE_LOG_WRITER, array('clsnam' => 'FILE_INDEX'));
        $this->log->write($message, Log::getDayKey(), FALSE, FALSE, $priority);

    }



    public function error($message) {

        $this->write($message, Log::EVENT_ERR);

    }

}

final class FileIndex {



    private static $log = FALSE;



    public static function create($fileid) {

        require_once('./../../framework/lib/util/Translator/Translator.inc');

        define('ASYNC_SERVICE', FALSE);
        define('SCRIPT_PATH', './../../');

        require_once('./config.inc');

        require_once('../../framework/globalVar.inc');
        include_once(LIB_PATH.'system/SysContext.inc');
        require_once('./localVar.inc');

        include(LIB_PATH.'util/JSON.inc');
        require_once(LIB_PATH.'system/SystemRegistry.inc');
        require_once(LIB_PATH.'system/JScript.inc');
        require_once(LIB_PATH.'system/GlobalSystemConfig.inc');
        include_once(LIB_PATH.'system/UserSystemConfig.inc');
        require_once(LIB_PATH.'system/Clipboard.inc');
        require_once(LIB_PATH.'system/UserRights.inc');
        require_once(LIB_PATH.'system/SysContext.inc');
        require_once(LIB_PATH.'util/Money.inc');
        require_once(FUNC_PATH.'GetUsrAcc.inc');
        require_once(LIB_PATH.'system.inc');
        require_once(LIB_PATH.'db/PgManager.inc');
        require_once(LIB_PATH.'util/Debug.inc');
        require_once(LIB_PATH.'db/Bean.inc');
        include_once(LIB_PATH.'filesystem/File.inc');
        include_once(LIB_PATH.'system/VarPathService.inc');
        include_once('classes/Converter/Converter.inc');
        require_once('./classes/Log/File/FileLog.inc');

        SysContext::$usr_info = array(
                'usr_id' => 1,
                'orunid' => '',
                'lasnam' => Translator::translate('CronRunner'),
                'firnam' => 'Cron',
                'usrnam' => 'CronRunner'
                );

        self::$log = new FileIndexLog();
        self::$log->write(sprintf('Odpalam FileIndexingTool::FileIndex dla fileid %s', $fileid));

        $db = new PgManager(DB_NAME);

        $ent_id = $db->select('podmioty', 'id_pod', 'symbol = \''.ENTITY.'\'', 'id_pod');
        SysContext::$ent_id = $ent_id;
        $_SESSION['curr_ent_id'] = $ent_id;

        $file = new File($fileid);
        if (!$file->URI) {
            self::$log->error(sprintf('Plik o ID %s (files.fileid) nie posiada uri', $fileid));
            return '';
        }

        $location = FileLocation::createByURI($file->URI);
        $uri = $location->get($file->URI);
        if (!file_exists($uri)) {
            self::$log->error(sprintf('Brak pliku w lokalizacji %s', $uri));
            return '';
        }

        if (!is_readable($uri)) {
            self::$log->error(sprintf('Brak prawa do odczytu pliku w lokalizacji %s', $uri));
            return '';
        }

        $mtyp = MimeType::get($uri, NULL, $file->type);

        $res = FALSE;
        if ($mtyp == 'text/plain') {
            $res = $db->update('files', array('idxdat' => date('Y-m-d H:i:s'), 'metada' => file_get_contents($uri)), 'fileid = '.$file->fileId);
            $file->addHistory(Translator::translate('Indeksowanie zawartości'), NULL, 1, 'INDEXED');
        } else {
            $tmpPath = VarPathService::getTmpPath();
            $previewFile = realpath($tmpPath).'/f'.$file->fileId.'_'.$file->size.'.txt';
            if (file_exists($previewFile)) {
                bs_unlink($previewFile);
            }

            $db->commit();
            try {
                clearstatcache();

                if (strpos($mtyp, 'image') !== 0) {
                    $converter = new Converter($mtyp, 'text/plain');
                    if (get_class($converter->getFilter()) == 'NullFilter') {
                        throw new Exception(sprintf('Brak możliwości konwersji z %s na %s', $mtyp, 'text/plain'));
                    } else {
                        $converter->setOutFile($previewFile);
                        try {
                            $convFile = $converter->convertFromFile($uri);
                            if (!file_exists($previewFile) OR !filesize($previewFile)) {
                                //throw new Exception('Wystąpił nieokreślony błąd podczas indeksowania');
                            }
                        } catch (Exception $e) {
                            //throw new Exception(sprintf('Błąd %s', $e->getMessage()));
                        }
                    }
                }

                clearstatcache();

                if (!file_exists($previewFile) OR !filesize($previewFile)) {
                    if (self::doOCR($uri, $previewFile, $fileid)) {
                        $res = $db->update('files', array('idxdat' => date('Y-m-d H:i:s'), 'metada' => file_get_contents($previewFile)), 'fileid = '.$file->fileId);
                        $file->addHistory(Translator::translate('Indeksowanie zawartości'), NULL, 1, 'INDEXED');
                    }
                } else {
                    $res = $db->update('files', array('idxdat' => date('Y-m-d H:i:s'), 'metada' => file_get_contents($previewFile)), 'fileid = '.$file->fileId);
                    $file->addHistory(Translator::translate('Indeksowanie zawartości'), NULL, 1, 'INDEXED');
                }
            } catch (Exception $e) {
                self::$log->error(sprintf('Wystąpił wyjątek podczas indeksowania dla pliku %s: %s', $uri, $e->getMessage()));
            }
        }

        if (file_exists($previewFile)) {
            bs_unlink($previewFile);
        }

        return $res;

    }



    private static function doOCR($filePath, $ocrPath, $fileid) {

        $dir = realpath(VarPathService::getTmpPath()).DIRECTORY_SEPARATOR.''.$fileid.'_ocr_'.time();
        $input = FALSE;

        try {
            if (is_dir($dir)) {
                bs_remove($dir, TRUE);
            }

            if (!bs_mkdir($dir)) {
                throw new Exception(sprintf('Nie można utworzyć katalogu %s', $dir));
            }

            if (preg_match('/pdf$/i', $filePath)) {
                $res = exec('pdfimages -j "'.$filePath.'" "'.$dir.DIRECTORY_SEPARATOR.'img"', $arr, $exitCode);
                if ($exitCode) {
                    throw new Exception(__CLASS__.'-	['.$exitCode.']:	'.$res.'');
                }
                $filesCount = 0;
                foreach (new DirectoryIterator($dir) as $entry) {
                    if (!$entry->isFile() || (strpos($entry->getFilename(), 'img') !== 0)) continue;
                    $filesCount++;
                }
                if ($filesCount > 3) {
                    $input = $dir.DIRECTORY_SEPARATOR.'image.tif';

                    $res = exec('convert "'.$dir.DIRECTORY_SEPARATOR.'img*.*" "'.$input.'"', $arr, $exitCode);
                    if ($exitCode || !file_exists($input)) {
                        throw new Exception(__CLASS__.'-	['.$exitCode.']:	'.$res.'');
                    }
                } else {
                    foreach (new DirectoryIterator($dir) as $entry) {
                        if (!$entry->isFile() || (strpos($entry->getFilename(), 'img') !== 0)) continue;
                        $input = $entry->getPathname();
                    }
                }
            }

            if (preg_match('/image\//', MimeType::get($filePath))) {
                $input = $filePath;
            }

            if (!$input) {
                return FALSE;
            }

            $prefix = $dir.DIRECTORY_SEPARATOR.'ocr';

            $res = exec('tesseract "'.$input.'" "'.$prefix.'" -l pol', $arr, $exitCode);
            if ($exitCode) {
                throw new Exception(__CLASS__.'-	['.$exitCode.']:	'.$res.'');
            }
            unlink($input);
            rename($prefix.'.txt', $ocrPath);

            return TRUE;
        } catch(Exception $e) {
            throw $e;
        }

        return FALSE;

    }

}

if (!isset($args[0]) OR !ctype_digit((string)$args[0])) {
    echo sprintf('Błędny parametr %s', $args[0]);
    exit;
}

FileIndex::create($args[0]);

?>